/*QuadPlayer Song Optimizer
Copyright (c) 2005 James Byerly

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include <fstream>
#include <vector>
#include "TI83PlusProgram.h"

bool TI83PlusProgram::ReadProgramFile(const char *p_FileName)
{
	std::ifstream InFile;
	InFile.open(p_FileName, std::ios::in | std::ios::binary);
	if(!InFile)
	{
		return false;
	}
	InFile.read((char *)&Signature, 8);
	InFile.read((char *)&Unknown1, 3);
	InFile.read((char *)&Comment, 42);
	InFile.read((char *)&FileLen, sizeof(FileLen));
	InFile.read((char *)&VarHeaderLen, sizeof(VarHeaderLen));
	InFile.read((char *)&VarDataLen, sizeof(VarDataLen));
	InFile.read((char *)&VarType, sizeof(VarType));
	InFile.read((char *)&VarName, 8);
	InFile.read((char *)&Unknown2, sizeof(Unknown2));
	InFile.read((char *)&DataLen, sizeof(DataLen));
	InFile.read((char *)&ProgramLen, sizeof(ProgramLen));
	ProgramData.resize(ProgramLen);
	InFile.read((char *)&(ProgramData[0]), ProgramLen);
	InFile.read((char *)&CheckSum, sizeof(CheckSum));
	if(!InFile)
	{
		return false;
	}
	return true;
}

void TI83PlusProgram::UpdateChecksum(void)
{
	CheckSum = (VarHeaderLen >> 8) & 0xFF;
	CheckSum += (VarHeaderLen & 0xFF);
	CheckSum += (VarDataLen >> 8) & 0xFF;
	CheckSum += (VarDataLen & 0xFF);
	CheckSum = CheckSum + WORD(VarType);
	CheckSum += (Unknown2 >> 8) & 0xFF;
	CheckSum += (Unknown2 & 0xFF);
	CheckSum += (DataLen >> 8) & 0xFF;
	CheckSum += (DataLen & 0xFF);
	CheckSum += (ProgramLen >> 8) & 0xFF;
	CheckSum += (ProgramLen & 0xFF);
	for(size_t DataIndex = 0; DataIndex < ProgramData.size(); ++DataIndex)
	{
		CheckSum = CheckSum + WORD(ProgramData[DataIndex]);
	}
}

bool TI83PlusProgram::WriteProgramFile(const char *p_FileName)
{
	std::ofstream OutFile;
	OutFile.open(p_FileName, std::ios::out | std::ios::binary);
	if(!OutFile)
	{
		return false;
	}
	ProgramLen = WORD(ProgramData.size());
	VarDataLen = ProgramLen + 2;
	DataLen = ProgramLen + 2;

	OutFile.write((char *)&Signature, 8);
	OutFile.write((char *)&Unknown1, 3);
	OutFile.write((char *)&Comment, 42);
	OutFile.write((char *)&FileLen, sizeof(FileLen));
	OutFile.write((char *)&VarHeaderLen, sizeof(VarHeaderLen));
	OutFile.write((char *)&VarDataLen, sizeof(VarDataLen));
	OutFile.write((char *)&VarType, sizeof(VarType));
	OutFile.write((char *)&VarName, 8);
	OutFile.write((char *)&Unknown2, sizeof(Unknown2));
	OutFile.write((char *)&DataLen, sizeof(DataLen));
	OutFile.write((char *)&ProgramLen, sizeof(ProgramLen));
	OutFile.write((char *)&(ProgramData[0]), ProgramLen);
	UpdateChecksum();
	OutFile.write((char *)&CheckSum, sizeof(CheckSum));
	if(!OutFile)
	{
		return false;
	}
	return true;
}