/*QuadPlayer Song Optimizer
Copyright (c) 2005 James Byerly

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef TI83PlusProgram__H
#define TI83PlusProgram__H

#include <vector>

typedef unsigned __int8 BYTE;
typedef unsigned __int16 WORD;
//Info from ticalc.org, except for Unknown2 = 0 which is new in 83+
struct TI83PlusProgram
{
	char Signature[8];//**TI83F*
	BYTE Unknown1[3];//1A 0A 00
	char Comment[42];//Arbitrary string, 0 padded
	WORD FileLen;//Length of bytes after this field

	//Checksum Starts Here
	WORD VarHeaderLen;//0D 00 = 13
	WORD VarDataLen;//Length of file from ProgramLen on = DataLen = ProgramLen + 2
	BYTE VarType;//5 for normal, 6 for protected
	char VarName[8];//Arbitrary string, 0 padded
	WORD Unknown2;//Apparently always 0

	WORD DataLen;//Length of file from ProgramLen on = VarDataLen = ProgramLen + 2
	WORD ProgramLen;//Length of Program

	typedef std::vector<BYTE> ProgramDataVec;
	ProgramDataVec ProgramData;//ProgramLen bytes
	//Checksum Ends Here

	WORD CheckSum;//Sum of bytes between the two comments above

	//helper functions
	bool ReadProgramFile(const char *p_FileName);
	bool WriteProgramFile(const char *p_FileName);
private:
	void UpdateChecksum(void);
};
#endif